#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stddef.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <signal.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <regex.h>
#include <errno.h>
#include <syslog.h>

/* MAILCMD should be a valid shell command for sending mail that
 * can use STDIN for input.				*/
#define MAILCMD "mail -s \"FIREWALL LOG DAEMON\" "
#define MAILTARGET "root"
/* the above two defines end up getting concatenated if email address is
 * not specified on the command line */

/* LOGFILE should be a named pipe (FIFO) that syslog will push
 * kernel logs through.
 * Create the fifo using the command:
 * 	mkfifo /var/log/kernelpipe
 * 
 * Edit /etc/syslog.conf and include a line that will push logs
 * through the pipe. Add the following line:
 * 	kern.info		|/var/log/kernelpipe
 *
 * Restart syslogd and you are ready to go!		*/
#define LOGFILE "/var/log/kernelpipe"

/* BUFFERSIZE is the default number of log entries to queue before
 * sending them by mail. Set this to whatever is reasonable for you.
 * This number can be overridden by a command line switch. */
#define BUFFERSIZE 10


#define PIDFILE "/var/run/firelogd.pid"
#define BUFFERFILE "/var/tmp/firelogd.buffer"
#define TEMPLATE "/etc/firelog.conf"
#define SVCFILE "/etc/iana-port-numbers"
#define ICMPFILE "/etc/iana-icmp-numbers"
#define USE_EXT_SVC 1
#define USE_STD_SVC 2

#define PROTO_ICMP 1
#define PROTO_TCP 6
#define PROTO_UDP 17
#define PROTO_AH 100
#define PROTO_ESP 200
#define PROTO_UNK 300

#define RECURSE 1
#define NO_RECURSE 0

struct context {	//TABLE_PAT subexp	CHAIN_PAT subexp
	char month[4];	// 1			// 1
	char day[3];	// 2			// 2
	char time[9];	// 3			// 3
	char msg[50];	// 4			// 5 "in" 4 "by" 18
	char in[5];	// 5			// 6
	char out[5];	// 6
	char mac[46];	// 7 (MAC=xx:xx...)
};
struct lvl3 {
	char srcip[16];		// 8		// 8
	char r_srcip[255];
	char dstip[16];		// 9		// 10
	char r_dstip[255];
	char iplen[6];		// 10		// 12
	char tos[5];		// 11		// 13
	char sflags[6];
	char prec[5];		// 12
	char pflags[4];
	char ttl[4];		// 13		// 16
	char id[6];		// 14		// 14
	char frag[145];		// 15		// 15
	char fflags[4];
};
struct pkt_ICMP {
	char type[4];		// 18, 21	// 9
	char code[4];		// 19, 22	// 11
	char info[128];
	int hastrigger;
	char trigger[1024];	// 23
	struct pktlog *recursed;
};
struct pkt_TCP {
	char srcpt[6];		// 25		// 9
	char r_srcpt[16];
	char dstpt[6];		// 26		// 11
	char r_dstpt[16];
	char window[6];		// 27
	char res[5];		// 28
	char flags[23];	// URG ACK PSH RST SYN FIN - 29		// 17
	char tflags[9];
};
struct pkt_UDP {
	char srcpt[6];		// 31		// 9
	char r_srcpt[16];
	char dstpt[6];		// 32		// 11
	char r_dstpt[16];
	char ulen[6];		// 33
};
struct pkt_AH {
	char ahdata[64];	// 37
};
struct pkt_ESP {
	char espdata[64];	// 35
};
struct pkt_UNK {
	char unkdata[64];	// 38		// 7
};
union lvl4 {
	struct pkt_ICMP pkt_ICMP;
	struct pkt_TCP pkt_TCP;
	struct pkt_UDP pkt_UDP;
	struct pkt_AH pkt_AH;
	struct pkt_ESP pkt_ESP;
	struct pkt_UNK pkt_UNK;
};
struct pktlog {
	int ptype;	// protocol type
	int ltype;	// log type (MAT_TABLE || MAT_CHAIN)
	char proto[11];	// protocol type
	struct context context;
	struct lvl3 lvl3;
	union lvl4 lvl4;
	char log[1024];
};


int match (const char *string, const regex_t * re, struct pktlog *pkt,
	   const int type, const int recurse);
void quit (int sig);		/* close it down properly */
int readbuf (void);		/* read in the buffer file */
int writebuf (void);		/* write buffer file */
int icmp_resolve (int icmptype, int icmpcode, char *icmpst);
int resolve_pkt (struct pktlog *pkt);
int fmt_prn (FILE * out, const struct pktlog *pkt, int preproc_c, char **preproc_v, const int recurse);
void pkt_str(const char *inst, char *outst, const struct pktlog *pkt);	/* returns string for
						   	string matching var name
							struct pktlog	*/
void usage (char *app);		/* usage and die */
